#!/usr/bin/env python3
""" Copyright start
  Copyright (C) 2008 - 2022 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end """
import os, sys
if os.getuid() != 0:
    print("This script must be run as 'root'")
    sys.exit(1)
import argparse, json, subprocess, re, shutil
sys.path.append("/opt/cyops/scripts/")
from csadm.logs.logger_defs import get_logger
MSG_EXECUTE_CMD = "Executing command [{0}]"
MSG_CMD_FAIL = "Failed to execute cmd [{0}] with return code as [{1}], output as [{2}] and error as [{3}]"
ENV_INTEGRATIONS = 'integrations'
VALID_ENVS = [ENV_INTEGRATIONS]
KEY_PIP = 'pip'
KEY_SITE_PACKAGES = 'site_packages'
ENV_DETAILS = {
    ENV_INTEGRATIONS: {
        KEY_PIP: '/opt/cyops-integrations/.env/bin/pip',
        KEY_SITE_PACKAGES: ['/opt/cyops-integrations/.env/lib/python3.6/site-packages']
    }
}
LOG_FILE = "/var/log/cyops/pip-cleanup.log"
logger = get_logger(__name__, LOG_FILE)

def byte2human(no_of_bytes):
    symbols = ('K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
    prefix = {}
    for i, s in enumerate(symbols):
        prefix[s] = 1 << (i + 1) * 10
    for s in reversed(symbols):
        if no_of_bytes >= prefix[s]:
            value = float(no_of_bytes) / prefix[s]
            return '%.1f%s' % (value, s)
    return "%B" % no_of_bytes

def execute_cmd(cmd, ignore_failure=False):
    pipes = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    out, err = pipes.communicate()
    ret = pipes.returncode
    out = out.strip().decode("utf-8")
    err = err.strip().decode("utf-8")
    if (ret != 0) and (not ignore_failure):
        logger.error(MSG_CMD_FAIL.format(cmd, ret, out, err))
        sys.exit(1)
    return (ret, out, err)

def get_installed_packages(pip_executable):
    format_cmd = "{0} list --format json"
    cmd = format_cmd.format(pip_executable)
    ret, out, err = execute_cmd(cmd)
    return json.loads(out.strip())

def get_older_packages(input_env_details, package_name, latest_package_version):
    list_site_packages = input_env_details[KEY_SITE_PACKAGES]
    package_name = package_name.replace("-", "_")
    files = []
    for site_packages in list_site_packages:
        tmp_files = [f for f in os.listdir(site_packages) if re.match(re.escape(package_name) + r'-' + '.+dist-info', f)]
        if tmp_files:
            files.extend(tmp_files)
    latest_package = package_name + "-" + latest_package_version + ".dist-info"
    files.remove(latest_package)
    if files:
        logger.info("Found older versions {0} for package {1}".format(files, latest_package))
    return files

def get_directory_size_in_bytes(path):
    return int(subprocess.check_output(['du','-sb', path]).split()[0].decode('utf-8'))

def process_env(input_env_details):
    installed_packages = get_installed_packages(input_env_details[KEY_PIP])
    list_site_packages = input_env_details[KEY_SITE_PACKAGES]
    cleanup_size_bytes = 0
    for installed_package in installed_packages:
        package_name = installed_package['name']
        package_latest_version = installed_package['version']
        older_packages = get_older_packages(input_env_details, package_name, package_latest_version)
        if older_packages:
            for old_package_name in older_packages:
                for site_packages in list_site_packages:
                    old_package_full_dir_path = site_packages + "/" + old_package_name
                    if os.path.isdir(old_package_full_dir_path):
                        cleanup_size_bytes = cleanup_size_bytes + get_directory_size_in_bytes(old_package_full_dir_path)
                        logger.info("Deleting directory {0}".format(old_package_full_dir_path))
                        shutil.rmtree(old_package_full_dir_path)
    if cleanup_size_bytes != 0:
        logger.info("Total cleaned up space {0}".format(byte2human(cleanup_size_bytes)))

def define_parser(parser):
    parser.add_argument('--env',
                        dest='input_env',
                        required=True,
                        choices=VALID_ENVS,
                        help='PIP environment')

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    define_parser(parser)
    args = parser.parse_args()
    process_env(ENV_DETAILS[args.input_env])
    sys.exit(0)